 /*--------------------------------------------------------------------
  * kmot.c - KoreBot Library - KoreMotor Support
  *--------------------------------------------------------------------
  * $Id: kmot.c,v 1.5 2005/10/25 12:44:56 pbureau Exp $
  *--------------------------------------------------------------------
  * Author: Alexandre Colot & Pierre Bureau
  *         heavy modifications by Yves Piguet to abstract from the
  *         I2C access layer
  * $Author: pbureau $
  * $Date: 2005/10/25 12:44:56 $
  * $Revision: 1.5 $
  *--------------------------------------------------------------------*
  * Modifications made by Yves Piguet (27 Feb 2004):
  *
  * - removed K*-style command handler (SpecsTable, Command_Table,
  *      cmd_handler, test_command_length, SyntaxError
  * - removed led_state
  * - removed scan_and_configure (seems incomplete)
  * - removed main
  * - removed last bits of Kbios- and compiler-specific code
  *      (#use, #fuse, #device, Fifo, #define OS_*)
  * - removed #include's I don't have (16F876A.h, registers.h,
  *      fifo.h, i2c.h, rs232.h, knet.h)
  * - removed write_i2c and read_i2c
  * - removed "Special Function available only for KNET"
  * - defined types i2c_io_readbyte_t, i2c_io_writebyte_t, i2c_io_t
  * - defined macros READI2C(i2c, slave, addr) and WRITEI2C(i2c,
  *      slave, addr, value)
  * - changed get_status, configure_pid, get_measure, set_position,
  *      set_profile, set_limits, search_limits, set_options, set_point,
  *      save_config
  * - removed  kmot_SearchLimits, kmot_GetStatus, kmot_ConfigurePID,
  *      kmot_GetMeasure, kmot_SetCurrentLimits, kmot_SetPositionLimits;
  *      renamed K commands above accordingly and recycled
  *      documentation comments
  * - changed kmot_ResetError, kmot_SetBlockedTime, kmot_SetSampleTime,
  *      kmot_SetFilterOrder to use i2c_io_t
  * - removed kmot_I2cRead, kmot_I2cWrite (not used anymore)
  * - changed kmot_I2cRead32 and kmod_I2cRead16 to use i2c_io_t
  * - removed kmot_TestLink
  *
  * $Log: kmot.c,v $
  * Revision 1.5  2005/10/25 12:44:56  pbureau
  * New function to read kmot options
  *
  * Revision 1.4  2004/11/12 14:35:23  pbureau
  * Minor corrections
  *
  * Revision 1.3  2004/09/02 14:04:00  cgaudin
  * Minor corrections.
  *
  * Revision 1.2  2004/08/16 20:05:22  cgaudin
  * Corrected bugs in kmot.c
  *
  * Revision 1.1  2004/07/29 10:51:55  cgaudin
  * New libkorebot release 1.2
  *
  *--------------------------------------------------------------------*/
#include <stdio.h>
 #include <korebot/korebot.h>
 //ӵ{X #include <korebot.h> ڥ䤣

 /*--------------------------------------------------------------------*/
 void kmot_GetFWVersion( knet_dev_t * dev ,
                         unsigned int * version )
 {
   unsigned char val;

   knet_read8( dev , MOT_FWVersion, &val );
   *version  = val;
 }

 /*--------------------------------------------------------------------*/
 void kmot_GetStatus( knet_dev_t * dev ,
                      unsigned char * status ,
                      unsigned char * error )
 {
   if ( status != NULL )
     knet_read8( dev , MOT_StatusFlags , status );

   if ( error != NULL )
     knet_read8( dev , MOT_ErrorFlags , error );
 }

 /*--------------------------------------------------------------------*/
 void kmot_GetOptions( knet_dev_t * dev ,
                      unsigned char * software,
                      unsigned char * hardware)
 {
   if ( hardware != NULL )
     knet_read8( dev , MOT_HWOptions, hardware);

   if ( software != NULL )
     knet_read8( dev , MOT_SWOptions, software);
 }

 /*--------------------------------------------------------------------*/
 void kmot_ConfigurePID( knet_dev_t * dev ,
                         int regtype ,
                         int16_t Kp , int16_t Kd , int16_t Ki )
 {
   unsigned char regs[3];

   switch(regtype) {

     /* Position */
   case kMotRegPos:
     regs[0] = MOT_KpPosL;
     regs[1] = MOT_KdPosL;
     regs[2] = MOT_KiPosL;
     break;

   case kMotRegPosProfile:   /* PosProfile   */
   case kMotRegSpeed:        /* Speed        */
   case kMotRegSpeedProfile: /* SpeedProfile */

     regs[0] = MOT_KpSpeedL;
     regs[1] = MOT_KdSpeedL;
     regs[2] = MOT_KiSpeedL;
     break;

   case kMotRegTorque: /* Current */
     regs[0] = MOT_KpTorqueL;
     regs[1] = MOT_KdTorqueL;
     regs[2] = MOT_KiTorqueL;
     break;
   }

   knet_set_order( dev , KMOT_ORDER_MASK );

   knet_write16( dev , regs[0] , Kp );
   knet_write16( dev , regs[1] , Kd );
   knet_write16( dev , regs[2] , Ki );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetPointSource( knet_dev_t * dev ,
                           int regtype ,
                           int wavetype ,
                           int period ,
                           int amplitude ,
                           int offset )
 {
   knet_write8( dev , MOT_SetPointSource , wavetype );
   knet_write8( dev , MOT_IntGenPeriod , period );
   knet_write8( dev , MOT_IntGenAmplitude , amplitude );
   knet_write8( dev , MOT_IntGenOffset , offset );
   knet_write8( dev , MOT_Mode , 1 );
   knet_write8( dev , MOT_ControlTyp , regtype );
 }

 /*--------------------------------------------------------------------*/
 long kmot_GetMeasure( knet_dev_t * dev , int regtype )
 {
   short s;
   long m = 0; int rc;

   knet_set_order( dev , KMOT_ORDER_MASK );

   switch(regtype) {

   default:
   case kMotRegOpenLoop: /* OpenLoop */
     m = 0;
     break;

   case kMotRegPos:        /* Position */
   case kMotRegPosProfile: /* PosProfile */
     rc=knet_read32( dev , MOT_PositionLL , &m );
     break;

   case kMotRegSpeed:     /* Speed */
   case kMotRegSpeedProfile:
     knet_read16( dev , MOT_SpeedLL , &s );
     m = (long)s;
     break;

   case kMotRegTorque:
     knet_read16( dev , MOT_TorqueL , &s );
     m = (long)s;
     break;
   }

   return m;
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetPosition( knet_dev_t * dev , long position )
 {
   knet_set_order( dev , KMOT_ORDER_MASK );
   knet_write32( dev , MOT_PositionLL , position );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetSpeedProfile( knet_dev_t * dev ,
                            int maxspeed ,
                            int acceleration )
 {
   knet_set_order( dev , KMOT_ORDER_MASK );
   knet_write16( dev , MOT_MaxSpeedL , maxspeed );
   knet_write8( dev , MOT_Acceleration , acceleration );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetLimits( knet_dev_t *dev ,
                      int regtype ,
                      long softStopMin , long softStopMax )
 {
   unsigned short v;

   knet_set_order( dev ,  KMOT_ORDER_MASK );

   switch(regtype) {

   case kMotRegPos:
   case kMotRegPosProfile:
     knet_write32( dev , MOT_SoftStopMinLL , softStopMin );
     knet_write32( dev , MOT_SoftStopMaxLL , softStopMax );
     break;

   case kMotRegTorque:
     /* Only MOT_SWCurrentLimitL is used */
     v = (unsigned short)softStopMax;
     knet_write16( dev , MOT_SWCurrentLimitL , v );
     break;

   default:
     break;
   }
 }

 /*--------------------------------------------------------------------*/
 int kmot_SearchLimits( knet_dev_t * dev ,
                        int8_t blockedTime ,
                        int32_t setPoint ,
                        int32_t * minpos ,
                        int32_t * maxpos ,
                        unsigned int timeout )
 {
   kb_time_t t;
   int ok;

   knet_set_order( dev , KMOT_ORDER_MASK );

   knet_write8( dev ,  MOT_BlockedTime , blockedTime );
   knet_write8( dev ,  MOT_ControlTyp , 3 );

   knet_write32( dev , MOT_SetPointLL , setPoint );

   knet_write8( dev , MOT_Mode , kMotModeSearchLimit );

   t = kb_getTime() + timeout;

   ok = 0;
   do {
     if ( knet_read( dev , MOT_Mode ) == 0 ) {
       ok = 1;
       break;
     }
     usleep( 10000 );
   }
   while ( kb_getTime() < t );

   if (!ok) return -1; /* TIMEOUT */

   knet_read32( dev , MOT_SoftStopMinLL , (unsigned long *)minpos );
   knet_read32( dev , MOT_SoftStopMaxLL , (unsigned long *)maxpos );

   return 0; /* OK */
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetOptions( knet_dev_t *dev , int hwOptions , int swOptions )
 {
   knet_write8( dev ,  MOT_HWOptions , hwOptions );
   knet_write8( dev ,  MOT_SWOptions , swOptions );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetPoint( knet_dev_t * dev , int regtype , long setPoint )
 {
   /*int chgmode = 0;*/

   knet_set_order( dev , KMOT_ORDER_MASK );

   if ( knet_read( dev , MOT_Mode ) != kMotModeNormal ) {
     knet_write8( dev , MOT_Mode , kMotModeNormal );
   }

   /*
     if( knet_read( dev , MOT_ControlTyp ) != regtype ) {
     knet_write8( dev ,  MOT_Mode , kMotModeStopMotor );
     chgmode = 1;
     }
     else {
     if( knet_read( dev ,  MOT_Mode ) != kMotModeNormal )
     knet_write8( dev ,  MOT_Mode , kMotModeNormal );
     }
   */

   knet_write8( dev ,  MOT_ControlTyp , regtype );
   knet_write32( dev , MOT_SetPointLL , setPoint );
   knet_write8( dev ,  MOT_SetPointSource , kMotPtSrcExtI2C );

   /*
       if( chgmode ) {
       knet_write8( dev , MOT_Mode , kMotModeNormal );
       }
   */
 }

 /*--------------------------------------------------------------------*/
 void kmot_SaveConfig( knet_dev_t * dev )
 {
   knet_write8( dev , MOT_Mode , 0x55 );
   knet_write8( dev , MOT_Mode , 0xAA );
   knet_write8( dev , MOT_Mode , kMotModeSaveE2PROM );
 }

 /*--------------------------------------------------------------------*/
 void kmot_ResetError( knet_dev_t * dev )
 {
   knet_write8( dev , MOT_ErrorFlags , 0 );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetBlockedTime( knet_dev_t * dev , int time )
 {
   knet_write8( dev , MOT_BlockedTime , time );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetSampleTime( knet_dev_t * dev , int sample )
 {
   knet_set_order( dev , KMOT_ORDER_MASK );
   knet_write8( dev , MOT_SampleTimeH , (sample & 0xFF00) >> 8 );
   knet_write8( dev , MOT_SampleTimeL , (sample & 0x00FF));
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetFilterOrder( knet_dev_t * dev , int order )
 {
   knet_write8( dev , MOT_Filter, order );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetMode( knet_dev_t * dev , int mode )
 {
   knet_write8( dev , MOT_Mode , mode );
 }

 /*--------------------------------------------------------------------*/
 void kmot_SetMargin( knet_dev_t *dev , int margin )
 {
   knet_write8( dev , MOT_NearTargetMargin , margin );
 }
